@@ -142,8 +142,6 @@ class ClusterConnectionPool(ConnectionPool):
 
         self.max_connections = max_connections or 2 ** 31
         self.max_connections_per_node = max_connections_per_node
-        self.get_connection_failures = 0
-        self.max_get_connection_failures = 10
 
         if connection_class == SSLClusterConnection:
             connection_kwargs['ssl'] = True  # needed in Redis init
@@ -244,17 +242,11 @@ class ClusterConnectionPool(ConnectionPool):
         """
         num_connections = self.count_all_num_connections(node)
         if num_connections >= self.max_connections:
-            self.get_connection_failures += 1
-            if self.get_connection_failures >= self.max_get_connection_failures:
-                log.info("reset connection pool")
-                self.disconnect()
-                self.reset()
             if self.max_connections_per_node:
                 raise RedisClusterException("Too many connection ({0}) for node: {1}".format(num_connections, node['name']))
 
             raise RedisClusterException("Too many connections")
 
-        self.get_connection_failures = 0
         self._created_connections_per_node.setdefault(node['name'], 0)
         self._created_connections_per_node[node['name']] += 1
         connection = self.connection_class(host=node["host"], port=node["port"], **self.connection_kwargs)
@@ -264,6 +256,20 @@ class ClusterConnectionPool(ConnectionPool):
 
         return connection
 
+    def drop_connection(self, node, connection):
+        """
+        The connection is unusable right after make_connection
+        We need to drop this connection and recreate a new one
+        """
+        try:
+            connection.close()
+        except Exception:
+            log.exception("drop connection is failed for {}".format(node))
+        finally:
+            self._created_connections_per_node[node['name']] -= 1
+            if self._created_connections_per_node[node['name']] < 0:
+                self._created_connections_per_node[node['name']] = 0
+
     def release(self, connection):
         """
         Releases the connection back to the pool
@@ -350,16 +356,18 @@ class ClusterConnectionPool(ConnectionPool):
             # Try to get connection from existing pool
             connection = self._available_connections.get(node["name"], []).pop()
         except IndexError:
-            try:
-                connection = self.make_connection(node)
-            except Exception:
-                log.exception("cannot get new connection")
-                raise ConnectionError('Cannot get a new connection')
+            connection = self.make_connection(node)
             server_type = node.get("server_type", "master")
             if server_type == "slave":
-                connection.send_command('READONLY')
-                if nativestr(connection.read_response()) != 'OK':
-                    raise ConnectionError('READONLY command failed')
+                try:
+                    connection.send_command('READONLY')
+                    if nativestr(connection.read_response()) != 'OK':
+                        raise ConnectionError('READONLY command failed')
+                except Exception:
+                    # if we catch any exception here, the connection is not usable
+                    log.exception("send_command readonly has error")
+                    self.drop_connection(node, connection)
+                    raise ConnectionError('Connection is unusable')
 
         self._in_use_connections.setdefault(node["name"], set()).add(connection)
 
